import logging
import shutil
import warnings
from pathlib import Path

from .profile_parser import BlankLine, Comment, OVPNConfig, Parameter
from .types import CipherStrength, ProviderExtensions, TLSVersion

logger = logging.getLogger(__name__)


def process_pia(config: OVPNConfig) -> None:
    """Adds necessary options to force AES-GCM connections to Private Internet Access.

    :param config: The already hardened OVPN profile
    """
    cipher_strength = config.cipher_strength()

    if cipher_strength == CipherStrength.STRONG:
        cipher_settings = [
            Parameter("cipher", "AES-256-GCM"),
            Parameter("data-ciphers", "AES-256-GCM:CHACHA20-POLY1305:AES-256-CBC"),
            Parameter("ncp-disable"),
        ]
    else:
        cipher_settings = [
            Parameter("cipher", "AES-128-GCM"),
            Parameter("data-ciphers", "AES-128-GCM:CHACHA20-POLY1305:AES-128-CBC"),
            Parameter("ncp-disable"),
        ]

    lines_to_insert = [
        BlankLine(),
        Comment("# Begin Paranoid OpenVPN changes"),
        *cipher_settings,
        Comment("# End Paranoid OpenVPN changes"),
        BlankLine(),
    ]

    # Insert the cipher settings where the old cipher setting was located, also need to clear out the previous settings
    cipher_loc = config.index("cipher")
    for param in cipher_settings:
        try:
            del config[param.name]
        except KeyError:
            pass

    for i, line in enumerate(lines_to_insert, start=cipher_loc):
        config.insert(i, line)


def process_profile(config: OVPNConfig, min_tls: TLSVersion, provider_ext: ProviderExtensions) -> None:
    """Completely processes one OVPN profile.

    :param config: OVPN config to modify.
    :param min_tls: Minimum TLS version to require.
    :param provider_ext: Flag to indicate which, if any, provider specific tweaks to apply.
    """
    cipher_strength = config.cipher_strength()

    if cipher_strength in [CipherStrength.STRONG, CipherStrength.MEDIUM]:
        security_settings = [
            Parameter("tls-version-min", "{} or-highest".format(min_tls.value)),
            Parameter(
                "tls-cipher",
                "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384:TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256:TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384",  # noqa: E501
            ),
            Parameter("tls-groups", "secp521r1:X448:secp384r1:secp256r1:X25519"),
            Parameter("tls-ciphersuites", "TLS_AES_256_GCM_SHA384:TLS_CHACHA20_POLY1305_SHA256"),
        ]
    else:
        security_settings = [
            Parameter("tls-version-min", "{} or-highest".format(min_tls.value)),
            Parameter(
                "tls-cipher",
                "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256:TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256:TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",  # noqa: E501
            ),
            Parameter("tls-groups", "secp256r1:X25519"),
            Parameter("tls-ciphersuites", "TLS_AES_128_GCM_SHA256:TLS_CHACHA20_POLY1305_SHA256"),
        ]

    lines_to_insert = [
        BlankLine(),
        Comment("# Begin Paranoid OpenVPN changes"),
        *security_settings,
        Comment("# End Paranoid OpenVPN changes"),
        BlankLine(),
    ]

    for security_setting in security_settings:
        try:
            del config[security_setting.name]
        except KeyError:
            pass

    for i, setting in enumerate(lines_to_insert, start=config.last_before_inline()):
        config.insert(i, setting)

    if provider_ext == ProviderExtensions.PIA:
        process_pia(config)

    if cipher_strength == CipherStrength.WEAK:
        warnings.warn("Profile has WEAK cipher strength!")


def process_profiles(src: Path, dest: Path, min_tls: TLSVersion, provider_ext: ProviderExtensions) -> None:
    """Completely processes one or more OVPN profiles.

    :param src: Path to local input file or directory containing OVPN profile(s).
    :param dest: Path to the output file (if `src` was a file) or directory (if `src`) was a directory.
    :param min_tls: Minimum TLS version to require.
    :param provider_ext: Flag to indicate which, if any, provider specific tweaks to apply.
    """
    if src.is_file():
        dest.parent.mkdir(parents=True, exist_ok=True)
        config = OVPNConfig.read(src)
        process_profile(config, min_tls, provider_ext)
        config.write(dest)
    elif src.is_dir():
        # If dest is relative to src, rglob will cause this script to infinitely reprocess its own output
        if dest.is_relative_to(src):
            raise ValueError("dest path cannot be relative to src path")

        for child in src.rglob("*.*"):
            dest_file = dest / child.relative_to(src)
            dest_file.parent.mkdir(parents=True, exist_ok=True)

            if child.suffix == ".ovpn":
                config = OVPNConfig.read(child)
                process_profile(config, min_tls, provider_ext)
                config.write(dest_file)
            else:
                shutil.copy(child, dest_file)
    else:
        raise ValueError(f"Source does not exist: {src}")



import argparse
import logging
import sys
from pathlib import Path

from .input_handlers import ResolveSource
from .main import process_profiles
from .types import ProviderExtensions, TLSVersion

logger = logging.getLogger(__name__)


def cli() -> None:
    """Main command-line entry point into the program. Parses options and invokes the rest of the program."""
    parser = argparse.ArgumentParser(description="Harden OpenVPN profiles from popular providers")
    parser.add_argument("source", help="Path or HTTP to zip file containing original OpenVPN profiles")
    parser.add_argument("dest", type=Path, help="Path to output file or directory")
    parser.add_argument(
        "--min-tls", choices=["1.0", "1.1", "1.2", "1.3"], default="1.3", help="Minimum TLS version to require"
    )
    parser.add_argument(
        "--logging", default="INFO", choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"], help="Desired log level"
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument("--pia", default=False, action="store_true", help="Add Private Internet Access fixes/hardening")

    args = parser.parse_args()

    logging.basicConfig(
        level=getattr(logging, args.logging), format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )

    if args.pia:
        provider_extensions = ProviderExtensions.PIA
    else:
        provider_extensions = ProviderExtensions.NONE

    try:
        with ResolveSource(args.source) as src:
            process_profiles(src, args.dest, TLSVersion(args.min_tls), provider_extensions)
            sys.exit(0)
    except Exception as exc:
        logging.critical("Failed processing source", exc_info=exc)
        sys.exit(1)



import re
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Iterable, Optional, Sequence, TextIO, Union

from .types import CipherStrength


class OVPNConfigParam(ABC):
    """ABC that represents a single setting in an OpenVPN config. Used mostly for `typing` purposes."""

    @abstractmethod
    def write(self, f_out: TextIO) -> int:  # pragma: no cover
        """Abstract method to require children to support writing their contents to a file."""
        pass

    @classmethod
    @abstractmethod
    def read(cls, config: Sequence[str]) -> "OVPNConfigParam":  # pragma: no cover
        """Abstract method to require children to implement a factory that reads from config lines."""
        pass

    @abstractmethod
    def __len__(self) -> int:  # pragma: no cover
        """Abstract method to require children to implement support for the len() function."""
        pass

    def __eq__(self, other: object) -> bool:  # pragma: no cover
        """Returns whether this object and another `OVPNConfigParam` are equal."""
        if not isinstance(other, OVPNConfigParam):
            return NotImplemented

        return self.name == other.name and self.value == other.value

    @property
    @abstractmethod
    def name(self) -> Union[str, None]:  # pragma: no cover
        """Abstract method to require children to expose the setting's name, if applicable."""
        pass

    @property
    @abstractmethod
    def value(self) -> Union[str, None]:  # pragma: no cover
        """Abstract method to require children to expose the setting's value, if applicable."""
        pass


class BlankLine(OVPNConfigParam):
    """Class that represents a literal blank line. Need this for method compatibility."""

    def write(self, f_out: TextIO) -> int:
        """Writes a blank line to the output OVPN profile.

        :param f_out: Handle to file open for writing.
        :return: The number of lines written.
        """
        f_out.write("\n")
        return 1

    @classmethod
    def read(cls, config: Sequence[str]) -> "BlankLine":
        """Factory function that returns an instance of this class if the head of `config` is applicable.

        :raises ValueError: Raised if the first config line isn't blank.
        :return: Instance of this class initialized with the head of `config`
        """
        if config[0].strip() != "":
            raise ValueError("Line is not empty")

        return BlankLine()

    def __len__(self) -> int:
        """Returns the number of lines this element takes up."""
        return 1

    @property
    def name(self) -> None:
        """Returns None as blank lines don't have a name."""
        return None

    @property
    def value(self) -> None:
        """Returns None as blank lines don't have a value."""
        return None


class Comment(OVPNConfigParam):
    """Class represending a OVPN profile comment."""

    def __init__(self, comment: str) -> None:
        """Constructor.

        :param comment: The content of the comment, prefixed with the comment character.
        """
        self.comment = comment.strip()

    def write(self, f_out: TextIO) -> int:
        """Writes the contents of the comment to the output OVPN profile.

        :param f_out: Handle to file open for writing.
        :return: The number of lines written.
        """
        f_out.write(f"{self.comment}\n")
        return len(self)

    @classmethod
    def read(cls, config: Sequence[str]) -> "Comment":
        """Factory function that returns an instance of this class if the head of `config` is applicable.

        :raises valueerror: raised if the first config line doesn't start with a comment character
        :return: Instance of this class initialized with the head of `config`
        """
        if not (config[0].startswith("#") or config[0].startswith(";")):
            raise ValueError("Line does not start with a comment character")

        return Comment(config[0])

    def __len__(self) -> int:
        """Returns the number of lines this element takes up."""
        return 1

    @property
    def name(self) -> str:
        """Returns the content of the comment."""
        return self.comment

    @property
    def value(self) -> None:
        """Returns None as comments don't have a value."""
        return None


class Inline(OVPNConfigParam):
    """Class represending a OVPN profile inline value (e.g. ca, cert)."""

    def __init__(self, param: str, value: Sequence[str]) -> None:
        """Constructor.

        :param param: The name of the inline parameter, including <>s
        :param value: Sequence of strings that make up the inline value
        """
        self._name = param.strip()
        self._value = [item.strip() for item in value]

    def write(self, f_out: TextIO) -> int:
        """Writes the contents of the inline element to the output OVPN profile.

        :param f_out: Handle to file open for writing
        :return: The number of lines written
        """
        naked_name = self._name[1:-1]
        f_out.write(f"<{naked_name}>\n")
        for line in self._value:
            f_out.write(f"{line}\n")
        f_out.write(f"</{naked_name}>\n")

        return len(self)

    @classmethod
    def read(cls, config: Sequence[str]) -> "Inline":
        """Factory function that returns an instance of this class if the head of `config` is applicable.

        :raises ValueError: Raised if the first config line isn't an inline tag
        :return: Instance of this class initialized with the entire contents of the inline tag
        """
        line = config[0].strip()

        tag_match = re.match(r"<([a-z0-9][a-z\-\_0-9]*[a-z0-9])>", line)
        if not tag_match:
            raise ValueError(f"Line is not an inline tag: {line}")

        param = tag_match.group(0)
        stripped_param = tag_match.group(1)
        value = []

        for line in config[1:]:
            if line.startswith(f"</{stripped_param}>"):
                break
            value.append(line)
        else:
            raise ValueError("Tag was never closed")

        return Inline(param, value)

    def __len__(self) -> int:
        """Returns the number of lines this element takes up."""
        return 2 + len(self._value)

    @property
    def name(self) -> str:
        """Returns the name of the inline tag, including the <>s."""
        return self._name

    @property
    def value(self) -> str:
        """Returns the the contents of the inline tag as one string."""
        return "\n".join(self._value)


class Parameter(OVPNConfigParam):
    """Class represending a standard OVPN profile parameter."""

    def __init__(self, param: str, value: Optional[str] = None) -> None:
        """Constructor.

        :param param: Name of the parameter
        :param value: Value of the parameter, can be None for some parameters
        """
        self._name = param.strip()
        self._value = value.strip() if value else None

    def write(self, f_out: TextIO) -> int:
        """Writes the contents of the parammeter to the output OVPN profile.

        :param f_out: Handle to file open for writing.
        :return: The number of lines written.
        """
        if self._value:
            f_out.write(f"{self._name} {self._value}\n")
        else:
            f_out.write(f"{self._name}\n")
        return 1

    @classmethod
    def read(cls, config: Sequence[str]) -> "Parameter":
        """Factory function that returns an instance of this class if the head of `config` is applicable.

        :return: Instance of this class initialized with the contents of the parameter
        """
        line = config[0].strip()

        if not re.match(r"[a-z0-9][a-z\-\_0-9]*[a-z0-9]", line):
            raise ValueError(f"Line is not a parameter: {line}")

        try:
            param, value = line.split(" ", maxsplit=1)
            return Parameter(param, value)
        except ValueError:
            return Parameter(line, None)

    def __len__(self) -> int:
        """Returns the number of lines this element takes up."""
        return 1

    @property
    def name(self) -> str:
        """Returns the name of the parameter."""
        return self._name

    @property
    def value(self) -> Optional[str]:
        """Returns the name of the parameter, can be None if the parameter has no value."""
        return self._value


class OVPNConfig:
    """Class that represents an entire OVPN profile."""

    def __init__(self, params: Optional[Iterable[OVPNConfigParam]] = None) -> None:
        """Constructor."""
        self.params = list(params) if params else []

    @classmethod
    def read(cls, config_file: Path) -> "OVPNConfig":
        """Factory function that returns an instance of this class based off the `config_file` input.

        :raises ValueError: Raised if an unknown parameter is encountered
        :return: Instance of this class initialized with the entire contents of file
        """
        config = OVPNConfig()

        with config_file.open("rt") as f_in:
            lines = f_in.readlines()

        while lines:
            for parser in [BlankLine, Comment, Inline, Parameter]:
                try:
                    # mypy does not deal with abstract class methods well, see mypy issue #6244
                    ele = parser.read(lines)  # type: ignore[attr-defined]
                    config.add(ele)
                    lines = lines[len(ele) :]
                    break
                except ValueError:
                    pass
            else:
                raise ValueError(f"Unknown config file line {lines[0]} in {config_file}")

        return config

    def add(self, new_param: OVPNConfigParam, exist_ok: bool = False) -> None:
        """Adds a new parameter to the config.

        If the parameter already exists and `exist_ok` is True, the old value will be replaced in place; if `exist_ok`
        is False, an KeyError will be raised.

        :param new_param: New parameter to add to the profile.
        :param exist_ok: Whether it is OK if the parameter already exists in the config.
        :raises KeyError: Raised if `new_param` already exists and `exist_ok` is False
        """
        if not (isinstance(new_param, BlankLine) or isinstance(new_param, Comment)):
            for i, existing in enumerate(self.params):
                if existing.name == new_param.name:
                    if exist_ok:
                        self.params[i] = new_param
                        break
                    else:
                        raise KeyError(f"{new_param.name} already present in config")
            else:
                self.params.append(new_param)
        else:
            self.params.append(new_param)

    def write(self, out_file: Path) -> int:
        """Writes the contents of the entire config to the output OVPN profile.

        :param f_out: Handle to file open for writing.
        :return: The number of lines written.
        """
        total_lines = 0
        with out_file.open("wt") as f_out:
            for param in self.params:
                total_lines += param.write(f_out)

        return total_lines

    def __contains__(self, key: str) -> bool:
        """Magic function that implements "in"; returns whether that parameter is present in the config.

        :param key: The parameter name or comment contents
        :raises TypeError: Raise if `key` is not a str
        :return: Whether the parameter is present
        """
        if not isinstance(key, str):
            raise TypeError("key must be a str")

        for param in self.params:
            if key == param.name:
                return True
        else:
            return False

    def __getitem__(self, key: Union[str, int]) -> OVPNConfigParam:
        """Magic function that implements object dereference.

        Implements a hybrid of sequence and mapping deference. If `key` is an `int`, returns the element as if this
        was a list. If `key` is a `str`, returns the element as if this was a dictionary.

        :param key: The integer location or name of the desired parameter
        :raises TypeError: Raise if `key` is not an `int` and is Falsey
        :raises KeyError: Raised if `key` is a `str` and that element does not exist
        :return: Specified element
        """
        if isinstance(key, int):
            return self.params[key]
        elif not key:
            raise TypeError("Empty key not allowed")

        for param in self.params:
            if key == param.name:
                return param
        else:
            raise KeyError(f"{key} does not exist")

    def __delitem__(self, key: Union[str, int]) -> None:
        """Magic function that implements object deletion.

        Implements a hybrid of sequence and mapping deletion. If `key` is an `int`, deletes the element at that index.
        If `key` is a `str`, deletes that parameter by name.

        :param key: The integer location or name of the desired parameter to delete
        :raises TypeError: Raise if `key` is not an `int` and is Falsey
        :raises KeyError: Raised if `key` is a `str` and that element does not exist
        """
        if isinstance(key, int):
            del self.params[key]
            return
        elif not key:
            raise TypeError("Empty key not allowed")

        for i, param in enumerate(self.params):
            if key == param.name:
                del self.params[i]
                break
        else:
            raise KeyError(f"{key} does not exist")

    def index(self, key: str, start: Optional[int] = None, end: Optional[int] = None) -> int:
        """Implements behavior similar to `list.index()`.

        :param key: The parameter name to search for
        :param start: The minimum index to start at, defaults to the start
        :param end: The maximum index to search until (exclusive), defaults to the end
        :raises TypeError: Raised if the index is Falsey
        :raises KeyError: Raised if the key does not exist
        :return: The index of the element, if present
        """
        if not key:
            raise TypeError("Empty key not allowed")

        start = start or 0
        end = end or len(self.params)

        for i, param in enumerate(self.params[start:end]):
            if key == param.name:
                return i
        else:
            raise KeyError(f"{key} does not exist")

    def insert(self, index: int, param: OVPNConfigParam) -> None:
        """Implements behavior similar to `list.insert`.

        :param index: The desired index to insert at
        :param param: The new OVPN element to add
        :raises KeyError: Raise if `param` already exists
        """
        if param.name and not isinstance(param, Comment) and param.name in self:
            raise KeyError(f"{param.name} already exists")

        self.params.insert(index, param)

    def last_before_inline(self) -> int:
        """Returns the last viable line before inline elements start.

        This method is based on an assumption that inline elements are at the end of the ffile and therefore it is
        desirable to insert non-inline elements before them. It returns the last insert-able line number before
        any inline elements start.

        :return: The last line before inline elements start
        """
        try:
            return min([i for i, param in enumerate(self.params) if isinstance(param, Inline)])
        except ValueError:
            return len(self.params)

    def cipher_strength(self) -> CipherStrength:
        """Uses a dumb heuristic to determine the strength of the "cipher" in the file.

        This is not very enlightened: 256-bit -> strong, 192-bit -> medium, 128-bit -> acceptable, and everything
        else is weak. This would break if algorithms are added that don't follow these common key sizes.

        :return: Enum value that describes the cipher stregnth
        """
        cipher = self["cipher"].value.upper() if self["cipher"].value else None

        if cipher and ("256" in cipher or "CHACHA20-POLY1305" in cipher):
            return CipherStrength.STRONG
        elif cipher and "192" in cipher:
            return CipherStrength.MEDIUM
        elif cipher and ("128" in cipher or "SEED-" in cipher or "SM4-" in cipher):
            return CipherStrength.ACCEPTABLE
        else:
            return CipherStrength.WEAK



import logging
import os
import re
import shutil
import sys
import tempfile
import urllib.request
import warnings
import zipfile
from contextlib import ExitStack
from pathlib import Path
from types import TracebackType
from typing import Optional, Type, Union

if sys.version_info >= (3, 8):
    from typing import Final, Literal
else:
    from typing_extensions import Final, Literal  # pragma: no cover


logger = logging.getLogger(__name__)


CHUNK_SIZE: Final = 8192


class HandleZip:
    """Context manager that transparently extracts ZIP files to a temporary location."""

    def __init__(self, src: Path) -> None:
        """Extracts the input ZIP file to a temporary location.

        :param src: Path to the input ZIP file.
        """
        self.temp_dir = Path(tempfile.mkdtemp())

        try:
            with zipfile.ZipFile(src) as f_in:
                f_in.extractall(self.temp_dir)
                logger.debug("Zip file extracted temporarily to %s", self.temp_dir)
        except Exception:
            shutil.rmtree(self.temp_dir)
            raise

    def __enter__(self) -> Path:
        """Context manager __enter__ function.

        :return: Resolved path to a local directory to process.
        """
        return self.temp_dir

    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> Literal[False]:
        """Cleans up any temporary files resulting from the ZIP extraction."""
        shutil.rmtree(self.temp_dir)
        return False


class HandleDownload:
    """Context manager that transparently downloads remote files to a temporary location."""

    def __init__(self, url: str) -> None:
        """Downloads the resource given by `url`.

        :param url: URL to download.
        :raises ValueError: Raised for non-HTTP(S) URLs and if the download fails.
        """
        if not (url.startswith("http://") or url.startswith("https://")):
            raise ValueError("Can only download files via HTTP")

        if url.startswith("http://"):
            warnings.warn("Downloading OpenVPN profiles over insecure connection")

        resp = urllib.request.urlopen(url)  # noqa: S310

        if resp.code != 200:
            raise ValueError(f"Could not download remote file, HTTP error code: {resp.code}")

        fd, file_name = tempfile.mkstemp()
        os.close(fd)
        self.temp_file = Path(file_name)

        try:
            with self.temp_file.open("wb") as f_out:
                while True:
                    data = resp.read(8192)
                    f_out.write(data)
                    if len(data) != CHUNK_SIZE:
                        break

            logger.debug("Downloaded file temporarily saved at %s", self.temp_file)
        except Exception:
            self.temp_file.unlink()
            raise

    def __enter__(self) -> Path:
        """Context manager __enter__ function.

        :return: Resolved path to a local file to process or extract.
        """
        return self.temp_file

    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> Literal[False]:
        """Cleans up any temporary files resulting from the download."""
        self.temp_file.unlink()
        return False


class ResolveSource:
    """Context manager that handles all the different input sources and provides a path to a local file/directory."""

    def __init__(self, src: Union[str, Path]) -> None:
        """Performs the needed combination of downloading/extracting source files.

        :param src: HTTP(S) URL or local path to the OVPN profiles to process.
        """
        self.exit_stack = ExitStack()

        src_as_str = str(src)
        if re.search(r"^[a-zA-Z0-9]+://", src_as_str):
            if src_as_str.startswith("http://") or src_as_str.startswith("https://"):
                logger.debug("Determined source was remote file, downloading")
                self.path = self.exit_stack.enter_context(HandleDownload(src_as_str))
            else:
                raise ValueError("Only HTTP(S) supported as remote protocol")
        else:
            self.path = Path(src)

        if self.path.is_file():
            try:
                self.path = self.exit_stack.enter_context(HandleZip(self.path))
            except zipfile.BadZipFile:
                # Make an assumption that our thing is a non-zip file
                pass
        elif not self.path.is_dir():
            raise ValueError("Path does not exist")

    def __enter__(self) -> Path:
        """Context manager __enter__ function.

        :return: Resolved path to either a local file or directory containing OVPN profile(s).
        """
        return self.path

    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> Literal[False]:
        """Cleans up all the temporary files created by calling child context managers."""
        self.exit_stack.close()
        return False



from enum import Enum, auto, unique


@unique
class TLSVersion(Enum):
    """Enum for desired minimum TLS version to require."""

    v1_0 = "1.0"
    v1_1 = "1.1"
    v1_2 = "1.2"
    v1_3 = "1.3"


@unique
class CipherStrength(Enum):
    """Enum to denote the relative cipher strength level of a OVPN profile."""

    WEAK = auto()
    ACCEPTABLE = auto()
    MEDIUM = auto()
    STRONG = auto()


@unique
class ProviderExtensions(Enum):
    """Enum for which, if any, provider-specific customization to perform."""

    NONE = auto()
    PIA = auto()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['paranoid_openvpn']

package_data = \
{'': ['*']}

extras_require = \
{':python_version < "3.8"': ['typing-extensions>=3.7.4.3,<4.0.0.0']}

entry_points = \
{'console_scripts': ['paranoid_openvpn = paranoid_openvpn.cli:cli']}

setup_kwargs = {
    'name': 'paranoid-openvpn',
    'version': '1.0.0',
    'description': 'Hardening script for OpenVPN client profiles',
    'long_description': '# Paranoid OpenVPN\n\nParanoid OpenVPN hardens OpenVPN profiles and provides additional optional\nprovider-specific fixes (e.g. Private Internet Access).\n\n## Usage\n\nWhen installed, Paranoid OpenVPN provides the `paranoid_openvpn` executable\nwhich comes with built-in help.  These are the common options:\n\n```console\n$ pip install paranoid-openvpn\n$ # usage: paranoid_openvpn [--min-tls {1.0,1.1,1.2,1.3}] [--pia] source dest\n$ # Process a remote zip file of OpenVPN profiles and apply PIA fixes\n$ paranoid_openvpn --pia https://www.privateinternetaccess.com/openvpn/openvpn-strong.zip /path/to/output_dir\n$ # Process one profile and allow TLS 1.2 (default is 1.3)\n$ paranoid_openvpn --min-tls 1.2 /path/to/input/profile.ovpn /path/to/output/hardened.ovpn\n```\n\n`source` above can be a remote zip, remote single profile, local zip, local\nsingle file, or local directory.\n\n## Hardening OpenVPN\n\nMost OpenVPN users are aware of the `cipher` and `hash` settings but that is\nusually the extent of security options that people modify. OpenVPN, however,\nhas two distinct channels that each have their own security settings: the\ncontrol and data channel. The `cipher` and `hash` settings apply only to the\ndata channel but OpenVPN exposes settings for the control channel as well.\nThe control channel is used to exchange keys that are then used to encrypt\nyour traffic in the data channel.\n\nParanoid OpenVPN tries to match the security of the data channel to the control\nchannel. In broad terms, OpenVPN has options for <128-bit, 128-bit, 192-bit,\nand 256-bit ciphers for the data channel. Paranoid OpenVPN will configure the\ncontrol channel to match these protection levels, with an absolute minimum of\n128-bits.\n\n## Cryptographic Reasoning\n\nWhere cryptographic judgement calls needed to be made, these rules were followed:\n\n  * [AEAD ciphers][aead] are always preferred over non-AEAD ciphers\n  * At the 256-bit security level, AES-GCM was preferred over CHACHA20-POLY1305\n    (for no particular reason).\n  * The 192-bit security level is rounded up to 256-bit as there are no 192-bit\n    TLS ciphers.\n  * At the 128-bit security level, CHACHA20-POLY1305 was the preferred fallback\n    for AES-128-GCM instead AES-128-CBC because it is an AEAD cipher.\n    AES-128-CBC is then the fallback for CHACHA20-POLY1305.\n\n[aead]: https://en.wikipedia.org/wiki/Authenticated_encryption\n\n## Provider-specific Fixes\n\nMost VPN providers work fine with "normal" OpenVPN profiles but some providers\nbenefit from a few tweaks.\n\n### Private Internet Access (PIA)\n\nPIA\'s provided OpenVPN profiles seemingly only support AES-128-CBC and\nAES-256-CBC as the `cipher` option.  However with a little coaxing, PIA will\nconnect using AES-256-GCM and AES-128-GCM. Use the `--pia` flag to allow\nyour client to client with these AEAD ciphers.\n\n## Donations\n\nIf you use this project and feel it\'s worth a donation, check out\n[GitHub Sponsors][ghs] or [Buy Me a Coffee][bmac].\n\n[ghs]: https://github.com/sponsors/Caligatio\n[bmac]: https://www.buymeacoffee.com/caligatio\n\n## Credit\n\nA lot of inspiration for this project was taken from https://blog.securityevaluators.com/hardening-openvpn-in-2020-1672c3c4135a.\n',
    'author': 'Brian Turek',
    'author_email': 'brian.turek@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Caligatio/paranoid-openvpn',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
